@@ -2,23 +2,26 @@
 Defines atcli's main functionality.
 """
 
-#!/usr/bin/env python
+# !/usr/bin/env python
 
 from __future__ import absolute_import, print_function, unicode_literals
 
 import argparse
 import audioop
+import json
 import math
 import multiprocessing
 import os
-from json import JSONDecodeError
 import subprocess
 import sys
 import tempfile
 import wave
+from typing import List, Pattern, Optional
 
-import json
 import requests
+
+from atcli.SRTParser import SRTParser
+
 try:
     from json.decoder import JSONDecodeError
 except ImportError:
@@ -52,10 +55,11 @@ def percentile(arr, percent):
     return low_value + high_value
 
 
-class FLACConverter(object): # pylint: disable=too-few-public-methods
+class FLACConverter(object):  # pylint: disable=too-few-public-methods
     """
     Class for converting a region of an input audio or video file into a FLAC audio file
     """
+
     def __init__(self, source_path, include_before=0.25, include_after=0.25):
         self.source_path = source_path
         self.include_before = include_before
@@ -66,7 +70,7 @@ class FLACConverter(object): # pylint: disable=too-few-public-methods
             start, end = region
             start = max(0, start - self.include_before)
             end += self.include_after
-            #delete=False necessary for running on Windows
+            # delete=False necessary for running on Windows
             temp = tempfile.NamedTemporaryFile(suffix='.flac', delete=False)
             program_ffmpeg = which("ffmpeg")
             command = [str(program_ffmpeg), "-ss", str(start), "-t", str(end - start),
@@ -83,10 +87,11 @@ class FLACConverter(object): # pylint: disable=too-few-public-methods
             return None
 
 
-class SpeechRecognizer(object): # pylint: disable=too-few-public-methods
+class SpeechRecognizer(object):  # pylint: disable=too-few-public-methods
     """
     Class for performing speech-to-text for an input FLAC file.
     """
+
     def __init__(self, language="en", rate=44100, retries=3, api_key=GOOGLE_SPEECH_API_KEY):
         self.language = language
         self.rate = rate
@@ -101,6 +106,7 @@ class SpeechRecognizer(object): # pylint: disable=too-few-public-methods
 
                 try:
                     resp = requests.post(url, data=data, headers=headers)
+                    # print(resp.content)
                 except requests.exceptions.ConnectionError:
                     continue
 
@@ -119,10 +125,11 @@ class SpeechRecognizer(object): # pylint: disable=too-few-public-methods
             return None
 
 
-class Translator(object): # pylint: disable=too-few-public-methods
+class Translator(object):  # pylint: disable=too-few-public-methods
     """
     Class for translating a sentence from a one language to another.
     """
+
     def __init__(self, language, api_key, src, dst):
         self.language = language
         self.api_key = api_key
@@ -136,14 +143,14 @@ class Translator(object): # pylint: disable=too-few-public-methods
             if not sentence:
                 return None
 
-            result = self.service.translations().list( # pylint: disable=no-member
+            result = self.service.translations().list(  # pylint: disable=no-member
                 source=self.src,
                 target=self.dst,
                 q=[sentence]
             ).execute()
 
             if 'translations' in result and result['translations'] and \
-                'translatedText' in result['translations'][0]:
+                    'translatedText' in result['translations'][0]:
                 return result['translations'][0]['translatedText']
 
             return None
@@ -156,12 +163,14 @@ def which(program):
     """
     Return the path for a given executable.
     """
+
     def is_exe(file_path):
         """
         Checks whether a file is executable.
         """
         return os.path.isfile(file_path) and os.access(file_path, os.X_OK)
-    #necessary to run on Windows
+
+    # necessary to run on Windows
     if os.name == "nt":
         program += ".exe"
     fpath, _ = os.path.split(program)
@@ -169,7 +178,7 @@ def which(program):
         if is_exe(program):
             return program
     else:
-        #looks for file in the script execution folder before checking on system path
+        # looks for file in the script execution folder before checking on system path
         current_dir = os.getcwd()
         local_program = os.path.join(current_dir, program)
         if is_exe(local_program):
@@ -203,7 +212,8 @@ def extract_audio(filename, channels=1, rate=16000):
     return temp.name, rate
 
 
-def find_speech_regions(filename, frame_width=4096, min_region_size=0.5, max_region_size=6): # pylint: disable=too-many-locals
+def find_speech_regions(filename, frame_width=4096, min_region_size=0.5,
+                        max_region_size=6):  # pylint: disable=too-many-locals
     """
     Perform voice activity detection on a given audio file.
     """
@@ -213,7 +223,7 @@ def find_speech_regions(filename, frame_width=4096, min_region_size=0.5, max_reg
     n_channels = reader.getnchannels()
     chunk_duration = float(frame_width) / rate
 
-    n_chunks = int(math.ceil(reader.getnframes()*1.0 / frame_width))
+    n_chunks = int(math.ceil(reader.getnframes() * 1.0 / frame_width))
     energies = []
 
     for _ in range(n_chunks):
@@ -242,7 +252,7 @@ def find_speech_regions(filename, frame_width=4096, min_region_size=0.5, max_reg
     return regions
 
 
-def generate_subtitles( # pylint: disable=too-many-locals,too-many-arguments
+def generate_subtitles(  # pylint: disable=too-many-locals,too-many-arguments
         source_path,
         output=None,
         concurrency=DEFAULT_CONCURRENCY,
@@ -250,16 +260,19 @@ def generate_subtitles( # pylint: disable=too-many-locals,too-many-arguments
         dst_language=DEFAULT_DST_LANGUAGE,
         subtitle_file_format=DEFAULT_SUBTITLE_FORMAT,
         api_key=None,
-    ):
+        generate_txt=False
+):
     """
     Given an input audio/video file, generate subtitles in the specified language and format.
     """
 
     if os.name != "nt" and "Darwin" in os.uname():
-        #the default unix fork method does not work on Mac OS
-        #need to use forkserver
-        if 'forkserver' != multiprocessing.get_start_method(allow_none=True):
-            multiprocessing.set_start_method('forkserver')
+        # the default unix fork method does not work on Mac OS
+        # need to use forkserver
+        start_method = multiprocessing.get_start_method(allow_none=True)
+        if 'forkserver' != start_method:
+            # multiprocessing.set_start_method('forkserver')
+            multiprocessing.set_start_method('spawn')
 
     audio_filename, audio_rate = extract_audio(source_path)
 
@@ -332,6 +345,8 @@ def generate_subtitles( # pylint: disable=too-many-locals,too-many-arguments
     with open(dest, 'wb') as output_file:
         output_file.write(formatted_subtitles.encode("utf-8"))
 
+    if generate_txt:
+        SRTParser.extractTextFromSRT(str(dest))
     os.remove(audio_filename)
 
     return dest
@@ -373,6 +388,10 @@ def main():
     """
     Run atcli as a command-line program.
     """
+    #
+    # # bug on macos, disable multiprocessing
+    # multiprocessing.freeze_support()
+
     parser = argparse.ArgumentParser()
     parser.add_argument('source_path', help="Path to the video or audio file to subtitle",
                         nargs='?')
@@ -394,7 +413,8 @@ def main():
                         action='store_true')
     parser.add_argument('--list-languages', help="List all available source/destination languages",
                         action='store_true')
-
+    parser.add_argument('-t', '--generate_txt', help='create a more human readable txt file in addition to the srt',
+                        action='store_true')
     args = parser.parse_args()
 
     if args.list_formats:
@@ -412,22 +432,114 @@ def main():
     if not validate(args):
         return 1
 
+    file_queue = set()
+
+    input_is_dir = os.path.isdir(args.source_path)
+    if input_is_dir:
+        print("input is directory, scanning files for candidates")
+        cValidator = CandidateValidator()
+        for root, dirs, files in os.walk(top=args.source_path):
+            for file in files:
+                current_file = os.path.join(root, file)
+                if cValidator(current_file):
+                    file_queue.add(current_file)
+    else:
+        file_queue.add(args.source_path)
+
+    # # TESTING, REMOVE THIS:
+    # import random
+    # file_queue = [random.choice(list(file_queue))]
+
     try:
-        subtitle_file_path = generate_subtitles(
-            source_path=args.source_path,
-            concurrency=args.concurrency,
-            src_language=args.src_language,
-            dst_language=args.dst_language,
-            api_key=args.api_key,
-            subtitle_file_format=args.format,
-            output=args.output,
-        )
-        print("Subtitles file created at {}".format(subtitle_file_path))
+        print(file_queue)
+        for item in file_queue:
+            try:
+                pass
+                # print(item)
+                subtitle_file_path = generate_subtitles(
+                    source_path=item,
+                    concurrency=args.concurrency,
+                    src_language=args.src_language,
+                    dst_language=args.dst_language,
+                    api_key=args.api_key,
+                    subtitle_file_format=args.format,
+                    output=args.output,
+                    generate_txt=args.generate_txt
+                )
+                print("Subtitles file created at {}".format(subtitle_file_path))
+            except KeyboardInterrupt:
+                raise
+
+            except Exception as ex:
+                print(f'exception! \n {ex}')
+                raise
     except KeyboardInterrupt:
         return 1
-
+    except Exception as ex:
+        print(ex)
+        return 1
     return 0
 
 
+class CandidateValidator(object):
+    valid_filetypes: List[Pattern[str]]
+
+    def __init__(self):
+        import re
+        self.valid_filetypes = [
+            re.compile(r'.+\.mp4$'),
+            re.compile(r'.+\.mkv$')
+        ]
+
+    def __call__(self, input_file) -> Optional[bool]:
+        try:
+            for pattern in self.valid_filetypes:
+                if pattern.match(input_file):
+                    # check if subtitle file already exists
+
+                    if os.path.exists(self._change_ext(input_file, '.srt')):
+                        print(f'skipping {self._change_ext(input_file, ".srt")} exists, skipping')
+                        return False
+                    elif os.path.exists(self._change_ext(input_file, '.transcript')):
+                        print(f'skipping {self._change_ext(input_file, ".transcript")} exists, skipping')
+                        return False
+                    elif os.path.exists(self._change_ext(input_file, '.txt')):
+                        print(f'skipping {self._change_ext(input_file, ".txt")} exists, skipping')
+                        return False
+                    else:
+                        return True
+                else:
+                    continue
+            # if no match by end of list, then return false
+            return False
+            # start, end = region
+            # start = max(0, start - self.include_before)
+            # end += self.include_after
+            # # delete=False necessary for running on Windows
+            # temp = tempfile.NamedTemporaryFile(suffix='.flac', delete=False)
+            # program_ffprobe = which("ffprobe")
+            # command = [str(program_ffprobe), "-ss", str(start), "-t", str(end - start),
+            #            "-y", "-i", self.source_path,
+            #            "-loglevel", "error", temp.name]
+            # use_shell = True if os.name == "nt" else False
+            # subprocess.check_output(command, stdin=open(os.devnull), shell=use_shell)
+            # read_data = temp.read()
+            # temp.close()
+            # os.unlink(temp.name)
+            # return read_data
+        except KeyboardInterrupt:
+            return
+
+    def _change_ext(self, file: str, new_ext: str):
+        ext_parts = os.path.splitext(file)
+        ext = ext_parts[len(ext_parts) - 1]  # get the last element of list (extension)
+        # base_parts = ext_parts[0:len(ext_parts) - 1 ]
+        basepath = ext_parts[0]
+        return str(basepath + new_ext)
+
+
+if __name__ == '__main__':
+    # main()
+    sys.exit(main())
 # if __name__ == '__main__':
-    # sys.exit(main())
+# sys.exit(main())
